import requests
from bs4 import BeautifulSoup


def get_urls_from_onetab(onetab):
    """
    Get video urls from a link to the onetab shared page.

    Args:
        onetab (str): Link to a onetab shared page.

    Returns:
        list: List of links to the videos.

    """
    html = requests.get(onetab).text
    soup = BeautifulSoup(html, 'lxml')

    divs = soup.findAll('div', {'style': 'padding-left: 24px; '
                                         'padding-top: 8px; '
                                         'position: relative; '
                                         'font-size: 13px;'})

    return [div.find('a').attrs['href'] for div in divs]



from .ipla import Ipla
from .polsatnews import PolsatNews
from .polskieradio import PolskieRadio
from .radiozet import RadioZet
from .rmf24 import Rmf24
from .tokfm import TokFm
from .tvn24 import Tvn24
from .tvpinfo import TvpInfo
from .tvpparlament import TvpParlament
from .vod import Vod
from .vodtvp import VodTVP
from .wp import Wp

EXTRACTORS = [
    Ipla,
    PolsatNews,
    PolskieRadio,
    RadioZet,
    Rmf24,
    TokFm,
    Tvn24,
    TvpInfo,
    TvpParlament,
    Vod,
    VodTVP,
    Wp
]

__all__ = ['EXTRACTORS']



import re
from urllib.parse import urlparse

import dateparser
import requests
from bs4 import BeautifulSoup

from rtv.exceptions import VideoIdNotMatchedError
from rtv.extractors.common import Extractor


class TvpInfo(Extractor):
    SITE_NAME = 'tvp.info'
    _VALID_URL = r'https?://(?:www\.)?tvp\.info/'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = self.get_article_url()

        # some data will be scraped from the article page
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')

        # and some data will be scraped from the player page
        self.video_id = self._extract_id()
        self.player_url = self.get_player_url()
        self.player_html = requests.get(self.player_url).text
        self.player_soup = BeautifulSoup(self.player_html, 'lxml')

    def _extract_id(self):
        pattern = re.compile(r'object_id=(?P<id>\d+)')
        match = pattern.search(self.html)

        if match:
            return match.group('id')
        else:
            raise VideoIdNotMatchedError

    def get_article_url(self):
        """
        Get the url of the TVP Info article itself, not the url of the preview with
        the 'Przejdź do artykułu' hyperlink.

        Returns:
            (str): Url of the article with the video.

        """
        html = requests.get(self.url).text
        soup = BeautifulSoup(html, 'lxml')
        div = soup.find('div', class_='more-back')

        if div:
            parsed_uri = urlparse(self.url)
            domain = '{uri.scheme}://{uri.netloc}'.format(uri=parsed_uri)
            suffix = div.find('a', href=True)['href'].strip()
            article_url = domain + suffix
            return article_url
        else:
            return self.url

    def get_player_url(self):
        """
        Get the url of the page containing embedded video player. The html of that page contains
        more detailed data about the video than the article page.
        """
        return f'http://www.tvp.info/sess/tvplayer.php?object_id={self.video_id}'

    def get_date(self):
        span = self.soup.find('span', class_='date')

        if span:
            date_str = span.text
            return dateparser.parse(date_str)

    def get_title(self):
        title = self.player_soup.find('meta', {'property': 'og:title'})['content']
        return title

    def get_showname(self):
        pattern = re.compile(
            r'\"SeriesTitle\",'
            r'\s*value:\s*'
            r'\"(?P<showname>.*?)\"'
        )
        match = pattern.search(self.player_html)

        if match:
            showname = match.group('showname')
            return showname or None

    def get_description(self):
        description = self.player_soup.find('meta', {'property': 'og:description'})['content']
        return description

    def extract(self):
        entries = [{
            'title': self.get_title(),
            'showname': self.get_showname(),
            'description:': self.get_description(),
            'date': self.get_date(),
            'url': self.url,
            'ext': 'mp4',
        }]

        return entries



import re
from datetime import datetime
from typing import Optional

from bs4 import BeautifulSoup

from rtv.extractors.common import (
    Extractor, GenericDescriptionMixin, GenericTitleMixin
)
from rtv.utils import get_ext


class Rmf24(GenericTitleMixin, GenericDescriptionMixin, Extractor):
    SITE_NAME = 'rmf24.pl'
    _VALID_URL = r'https?://(?:www\.)?rmf24\.pl/'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')
        # TODO: use decorators to mark the need to use bs4?
        # TODO: make_soup(html=Optional) or/and load_html(url=Optional)

    def get_date(self) -> Optional[datetime]:
        meta_tag = self.soup.select_one('meta[itemprop=datePublished]')
        if meta_tag:
            date_published_str = meta_tag.get('content')
            return datetime.strptime(date_published_str, '%Y-%m-%dT%H:%M:%S')
        return None

    @staticmethod
    def extract_entry(scraped_info):
        """
        Transform scraped_info dictionary into an entry, under the assumption that there is only
        one track in 'track' list, since each video/audio is instantiated individually
        on the RMF website and each of them is scraped independently, so there shouldn't be cases
        when there are 2 unrelated tracks in one info_dict.

        Args:
            scraped_info (dict): Video info dict, scraped straight from the website.

        Returns:
            dict: Entry containing title, formats (url, quality), thumbnail, etc.

        """
        quality_mapping = {  # ascending in terms of quality
            'lo': 0,
            'hi': 1
        }

        entry = scraped_info['tracks'][0]
        '''
        The structure of entry is as follows:

        'src': {
            'hi': [
                {
                'src': 'http://v.iplsc.com/30-11-gosc-marek-jakubiak/0007124B3CGCAE6P-A1.mp4',
                'type': 'video/mp4'
                }
            ],
            'lo': [
                {
                'src': 'http://v.iplsc.com/30-11-gosc-marek-jakubiak/0007124B3CGCAE6P-A1.mp4',
                'type': 'video/mp4'
                }
            ]
        }
        '''

        sources = entry.pop('src')

        # TODO: #LOW_PRIOR Remove date from title of audio files e.g. '10.06 Gość: Jarosław Gowin'

        formats = []
        for src_name, src in sources.items():
            url = src[0]['src']
            formats.append({
                'url': url,
                'quality': quality_mapping[src_name],
                'ext': get_ext(url),
                'width': int(scraped_info.get('width', 0)),
                'height': int(scraped_info.get('height', 0)),
            })

        # outer level url and ext come from the video of the lowest quality
        # you can access rest of the urls under 'formats' key
        worst_format = min(formats, key=lambda f: f['quality'])
        entry.update({
            **entry.pop('data'),
            'formats': formats,
            'url': worst_format['url'],
            'ext': worst_format['ext']
        })

        return entry

    def _scrape_entries(self):
        entries = []

        pattern = re.compile(r'Video.createInstance\((?P<js_object>{.*?})\);', re.DOTALL)
        scripts = self.soup.findAll('script', text=pattern)

        for script in scripts:
            matches = pattern.findall(script.text)
            for data in matches:  # matches is a list of matched strings, not match objects
                info_dict = js2py.eval_js(f'Object({data})').to_dict()
                entries.append(self.extract_entry(info_dict))

        # temporarily return only audio entries if present, otherwise return all video entries
        audio_entries = [e for e in entries if e.get('type', 'video') == 'audio']
        if audio_entries:
            entries = audio_entries

        return entries

    def extract(self):
        audio_url = self._get_audio_source_url()
        extension = get_ext(audio_url)

        entries = [{
            'title': self.get_title(),
            'description': self.get_description(),
            'date': self.get_date(),
            'url': audio_url,
            'ext': extension
        }]

        return entries



import datetime
import re
from typing import Dict

import js2py
from bs4 import BeautifulSoup
from dateparser.search import search_dates

from rtv.extractors.common import Extractor, GenericDescriptionMixin


class PolsatNews(GenericDescriptionMixin, Extractor):
    SITE_NAME = 'polsatnews.pl'
    _VALID_URL = r'https?://(?:www\.)?polsatnews\.pl/'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')
        self.data = self._extract_data()

    def _extract_data(self) -> Dict[str, str]:
        pattern = re.compile(r'(?P<data>customPackage\s*=\s*\[.*?\])', re.DOTALL)
        match = pattern.search(self.html)

        if match:
            data_raw = match.group('data')
            data_list = js2py.eval_js(data_raw)
            data_dict = {d['name'].lower(): d['value'] for d in data_list}
            return data_dict
        return {}

    def get_show_name(self):
        show_name = self.data.get('series')
        return show_name

    def get_title(self):
        title = self.data.get('title')
        return title

    def get_date(self):
        # TODO: refactor, handle if parsing fails, come up with a more generic solution
        div = self.soup.select_one('div.article-meta-data div.fl-right')
        if div:
            date_str = div.text
            date = datetime.datetime.strptime(date_str, '%Y-%m-%d, %H:%M')
            return date

        description = self.get_description()
        found_dates = search_dates(description, languages=['pl'])
        if found_dates:
            _, date = found_dates[0]
            return date

        return None

    def extract(self):
        entries = [{
            'title': self.get_title(),
            'description': self.get_description(),
            'show_name': self.get_show_name(),
            'date': self.get_date(),
            'url': self.url,
            'ext': 'mp4'
        }]

        return entries



from datetime import datetime

import dateparser
from bs4 import BeautifulSoup

from rtv.extractors.common import Entries, Extractor


class Tvn24(Extractor):
    SITE_NAME = 'tvn24.pl'
    _VALID_URL = r'https?://(?:www\.)?(?:.*\.)?tvn24\.pl/.*'

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')

    def get_date(self) -> datetime:
        div = self.soup.select_one('div.articleDateContainer')
        time_tag = div.find('time', datetime=True)
        date_str = time_tag['datetime'].strip()
        time_str = div.find('span').text

        date = dateparser.parse(date_str)
        time = dateparser.parse(time_str).time()

        full_date = datetime.combine(date, time)
        return full_date

    def extract(self) -> Entries:
        entry = self.get_info()
        entry['date'] = self.get_date()
        return [entry]



from datetime import datetime
from typing import List, Optional, Tuple

import dateparser
import requests
from bs4 import BeautifulSoup

from rtv.extractors.common import Entries, Extractor


class Wp(Extractor):
    SITE_NAME = 'wp.pl'
    _VALID_URL = r'https?://(?:www\.)?video\.wp\.pl/.*'

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')
        self.data = self._fetch_data()

    @staticmethod
    def _get_json_url(mid: int) -> str:
        json_url = f'https://video.wp.pl/api/v1/embed/{mid}'
        return json_url

    def _fetch_data(self):
        mid = self.soup.select_one('#mainPlayer')['data-mid']
        json_url = self._get_json_url(mid)
        video_data = requests.get(json_url).json()['clip']
        return video_data

    def get_title(self) -> Optional[str]:
        title = self.data.get('title')
        return title

    def get_description(self) -> Optional[str]:
        description = self.data.get('description')
        return description

    def get_tags(self) -> List[str]:
        tags = self.data.get('tags', '').split(',')
        return tags

    def get_show_name(self) -> Optional[str]:
        show_name = self.data['media'].get('program')
        return show_name

    def get_date(self) -> Optional[datetime]:
        raw_date = self.data['media'].get('createDate')
        if raw_date:
            date = dateparser.parse(raw_date)
            return date
        return None

    @staticmethod
    def quality_comparator(video_data):
        """Custom comparator used to choose the right format based on the resolution."""
        def parse_resolution(res: str) -> Tuple[int, ...]:
            return tuple(map(int, res.split('x')))

        raw_resolution = video_data['resolution']
        resolution = parse_resolution(raw_resolution)
        return resolution  # e.g (1024, 576)

    def get_real_url(self) -> str:
        formats = list(filter(lambda d: d['type'] == 'mp4@avc', self.data['url']))  # filter out mp4
        worst_format = min(formats, key=self.quality_comparator)  # for now take the lowest quality
        url = worst_format['url']
        return url

    def extract(self) -> Entries:
        entries = [{
            'title': self.get_title(),
            'description': self.get_description(),
            'tags': self.get_tags(),
            'show_name': self.get_show_name(),
            'date': self.get_date(),
            'url': self.get_real_url(),
            'ext': 'mp4'
        }]
        return entries



import datetime
import json
import re

import requests
from dateparser.search import search_dates
from bs4 import BeautifulSoup

from rtv.exceptions import VideoIdNotMatchedError
from rtv.extractors.common import Extractor
from rtv.utils import get_ext


class Ipla(Extractor):
    SITE_NAME = 'ipla.tv'
    _VALID_URL = r'https?://(?:www\.)?ipla\.tv/.*/(?P<id>[0-9a-fA-F]+)'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.video_id = self._extract_id()
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')
        self.data = self._fetch_data()

    @staticmethod
    def _generate_client_id(length):
        import binascii
        import os

        return binascii.b2a_hex(os.urandom(length // 2))

    def _extract_id(self):
        # TODO: move to extractor.common? maybe merge with validate_url?
        match = re.search(self._VALID_URL, self.url)

        if match:
            return match.group('id')
        else:
            raise VideoIdNotMatchedError

    def _fetch_data(self):
        url = 'https://b2c.redefine.pl/rpc/navigation/'
        data = {
            'jsonrpc': '2.0',
            'id': 1,
            'method': 'prePlayData',
            'params': {
                'ua': 'www_iplatv_html5/12345',
                # 'ua': 'www_iplatv_html5/12345 (Windows 10; widevin=true)',
                #  can be used to get dash manifest
                'cpid': 1,
                'mediaId': self.video_id
            }
        }

        video_data = requests.post(url, data=json.dumps(data)).json()
        return video_data

    def get_date(self):
        description_tag = self.soup.select_one('.description-content__paragraph')
        description = description_tag.get_text(strip=True)
        found_dates = search_dates(description, languages=['pl'])

        if found_dates:
            _, date = found_dates[0]
        else:
            # TODO: handle if not present
            date_str = self.data['result']['reporting']['gastream']['premiereDate']
            date = datetime.datetime.strptime(date_str, '%Y%m%d')
        return date

    def get_title(self):
        # soup = BeautifulSoup(self.html, 'lxml')
        # title = soup.find('h1', class_='vod-title__content').text
        title = self.data['result']['reporting']['gastream']['title']  # TODO: handle if not present
        return title

    def get_showname(self):
        # TODO: handle if not present
        showname = self.data['result']['reporting']['gastream']['series']
        return showname

    def get_real_url(self):
        sources = self.data['result']['mediaItem']['playback']['mediaSources']
        source = min(sources, key=lambda s: s['quality'][:-1])  # choose lowest quality, e.g. 576p
        # TODO: handle all qualities
        return source['url']

    def extract(self):
        url = self.get_real_url()

        entries = [{
            'title': self.get_title(),
            'showname': self.get_showname(),
            'date': self.get_date(),
            'url': url,
            'ext': get_ext(url),
        }]

        return entries



import datetime
import re

import js2py
import requests
from bs4 import BeautifulSoup

from rtv.extractors.common import Extractor


class TvpParlament(Extractor):
    SITE_NAME = 'tvpparlament.pl'
    _VALID_URL = r'https?://(?:www\.)?tvpparlament\.pl/'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')
        self.data = self._fetch_data()

    @staticmethod
    def _get_json_url(*, object_id, sdt_version, **kwargs):
        json_url = (
            f'http://www.tvpparlament.pl/shared/cdn/tokenizer_v2.php'
            f'?object_id={object_id}'
            f'&std_version={sdt_version}'
        )
        return json_url

    def _fetch_data(self):
        pattern = re.compile(r'(?P<video_ids>playVideo\s*=\s*{.*})', re.DOTALL)
        script = self.soup.find('script', text=pattern)
        match = pattern.search(script.text)

        video_ids_raw = match.group('video_ids')
        video_ids = js2py.eval_js(f'Object({video_ids_raw})').to_dict()

        json_url = self._get_json_url(**video_ids)
        video_data = requests.get(json_url).json()

        return video_data

    def get_date(self):
        pattern = re.compile(r'[dD]ata:?\s+(?P<date>\d{2,4}-\d{1,2}-\d{1,2})')
        div = self.soup.find('div', text=pattern)
        match = pattern.search(div.text)

        date_str = match.group('date')
        date = datetime.datetime.strptime(date_str, '%Y-%m-%d')

        return date

    def get_show_name(self):
        # TODO: scrape show_name from web
        pattern = re.compile(r'https?://(?:www\.)?tvpparlament\.pl/(?P<show_name>[\w\-.,]+)/')
        match = pattern.match(self.url)
        return match.group('show_name').replace('-', ' ').title()

    def extract(self):
        # choose format providing manifest as it gives much flexibility in terms of quality choice
        entry = next((item for item in self.data['formats']
                      if item['mimeType'] == 'application/vnd.ms-ss'), None)

        entries = [{
            'title': self.data.get('title'),
            'show_name': self.get_show_name(),
            'date': self.get_date(),
            'url': entry['url'],
            'ext': 'mp4',
        }]

        return entries



import datetime
import json
import re

from bs4 import BeautifulSoup

from rtv.extractors.common import Extractor


class VodTVP(Extractor):
    SITE_NAME = 'vod.tvp.pl'
    _VALID_URL = (
        r'https?://(?:www\.)?vod.tvp\.pl/'
        r'.*?'
        r'(?:,(?P<date>[\d\-]+))?'
        r','
        r'(?P<object_id>\d+)'
    )
    # TODO: change _VALID_URLs to just plain domain name and move date, showname, etc. regexes
    # to functions, so the downloader matches the url despite of absence of additional
    # parameters in the url then the get_x functions will just return None and the Formatter
    # will take care of default values

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')

    def get_date(self):
        match = re.match(self._VALID_URL, self.url)

        date_str = match.group('date')
        date_formats = [
            '%d%m%Y',
            '%d%m%Y-%H%M'
        ]
        for d in date_formats:
            try:
                return datetime.datetime.strptime(date_str, d)
            except (ValueError, AttributeError):
                pass

    def get_show_name(self):
        """
        Get video show name from the website. It's located in the div with 'data-hover'
        attribute under the 'title' key.

        Returns:
            str: Video show name.

        """
        div = self.soup.find('div', attrs={'data-hover': True})
        data = json.loads(div['data-hover'])
        show_name = data.get('title')

        return show_name

    def get_title(self):
        """
        Get Video title from the website. It's located in the div with 'data-hover'
        attribute under the 'episodeCount' key.

        Returns:
            str: Video title.

        """
        # considered as a worse solution since most of the videos have only date in the title
        # soup = BeautifulSoup(self.html, 'lxml')
        # div = soup.find('div', attrs={'data-hover': True})
        # data = json.loads(div['data-hover'])
        # title = data.get('episodeCount')

        # TODO: _og_search_title/_og_search_description common method
        title = self.soup.find('meta', {'property': 'og:title'})['content']
        return title

    def get_description(self):
        """
        Get video description from the website. It's located in the meta tag
        with 'og:description' attribute under 'content' attribute.

        Returns:
            str: Video description.

        """
        description = self.soup.find('meta', {'property': 'og:description'})['content']
        return description

    def extract(self):
        entries = [{
            'title': self.get_title(),
            'show_name': self.get_show_name(),
            'description': self.get_description(),
            'date': self.get_date(),
            'url': self.url,
            'ext': 'mp4'
        }]

        return entries



import datetime
import re

from bs4 import BeautifulSoup

from rtv.extractors.common import Extractor


class Vod(Extractor):
    SITE_NAME = 'vod.pl'
    _VALID_URL = r'https?://(?:www\.)?vod\.pl/'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')
        self.headline_match = self._extract_headline()

    def get_date(self):
        # TODO: use better date regex / use dateparser package?
        # "uploadDate": "2018-02-08 12:14:22+0100"
        match = re.search(
            r'\"uploadDate\"'
            r'\s*:\s*' 
            r'\"(?P<date>\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}\+\d{4})\"'
            r'.*',
            self.html)

        if match:
            date_str = match.group('date')
            return datetime.datetime.strptime(date_str, '%Y-%m-%d %H:%M:%S%z')

    def _extract_headline(self):
        div = self.soup.find('div', class_='v_videoTitle')

        if div:
            headline = div.find('h2').find(text=True).strip()

            pattern = re.compile(
                r'^(?:(?P<show_name>[\w#\-.,\s]+?)\.?:\s*)?'  # optional show name
                r'(?P<title>\b[\w#\-.,\s]+\b)'
                r'.*$'
            )
            return pattern.match(headline)

        return None

    def get_show_name(self):
        match = self.headline_match
        if match:
            return match.group('show_name')

    def get_title(self):
        match = self.headline_match
        if match:
            return match.group('title')

    def extract(self):
        entries = [{
            'title': self.get_title(),
            'show_name': self.get_show_name(),
            'date': self.get_date(),
            'url': self.url,
            'ext': 'mp4',
        }]

        return entries



import re
from typing import Any, ClassVar, Dict, List, Match, Optional

import requests
import youtube_dl
from bs4 import BeautifulSoup

from rtv.utils import get_domain_name, suppress_stdout
from rtv.video import Video

Entry = Dict[str, Any]
Entries = List[Entry]


class Extractor:
    SITE_NAME: ClassVar[str]
    _VALID_URL: ClassVar[str]

    url: str
    html: str
    videos: List[Video]
    response: requests.models.Response

    def __init__(self, url: str) -> None:
        self.url = url
        self.videos = []

    @classmethod
    def validate_url(cls, url: str) -> Optional[Match[str]]:
        """Check if the Extractor can handle the given url."""
        match = re.match(cls._VALID_URL, url)
        return match

    def load_html(self) -> None:
        r = requests.get(self.url)
        r.encoding = 'utf-8'
        self.response = r
        self.html = r.text

    def get_info(self) -> dict:
        """Get information about the videos from YoutubeDL package."""
        with suppress_stdout():
            with youtube_dl.YoutubeDL() as ydl:
                info_dict = ydl.extract_info(self.url, download=False)
                return info_dict

    @staticmethod
    def update_entries(entries: Entries, data: dict) -> None:
        """Update each entry in the list with some data."""
        # TODO: Is mutating the list okay, making copies is such a pain in the ass
        for entry in entries:
            entry.update(data)

    def extract(self) -> Entries:
        """Extract data from the url. Redefine in subclasses."""
        raise NotImplementedError('This method must be implemented by subclasses')

    def run(self) -> None:
        entries = self.extract()

        self.update_entries(entries, {
            'site': get_domain_name(self.url)
        })

        if not isinstance(entries, list):
            raise TypeError('extract method must return an iterable of dictionaries')

        for entry in entries:
            video = Video(entry)
            self.videos.append(video)


class GenericTitleMixin:
    soup: BeautifulSoup

    def get_title(self) -> Optional[str]:
        meta_tag = self.soup.select_one('meta[property="og:title"]')
        if meta_tag:
            title = meta_tag.get('content')
            return title
        return None


class GenericDescriptionMixin:
    soup: BeautifulSoup

    def get_description(self) -> Optional[str]:
        meta_tag = self.soup.select_one('meta[property="og:description"]')
        if meta_tag:
            description = meta_tag.get('content')
            return description
        return None



import datetime
import re

from bs4 import BeautifulSoup

from rtv.extractors.common import Extractor
from rtv.utils import delete_duplicates


class RadioZet(Extractor):
    SITE_NAME = 'radiozet.pl'
    _VALID_URL = r'https?://(?:www\.)?radiozet\.pl/.*/(?P<show_name>[\w\-.,]+)/(?P<title>[\w\-.,]+)'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')

    def get_entries(self):
        # manifests available:
        # data-source-ss
        # data-source-dash
        # data-source-hls
        divs = delete_duplicates(item for item in self.soup.find_all()
                                 if 'data-source-dash' in item.attrs)

        # sort divs by id, first cast it to int
        # e.g. data-storage-id="90164"
        divs.sort(key=lambda u: int(u['data-storage-id']))

        # extract urls of dash manifests
        manifest_urls = [div['data-source-dash'] for div in divs]

        entries = [{
            'url': url,
            'title': self.get_title(),
            'show_name': self.get_show_name(),
            'date': self.get_date(),
            'ext': 'mp4'
        } for url in manifest_urls]

        # radiozet videos are usually divided into 2 or more videos,
        # a suffix is appended to the title, so there are no duplicated titles
        if len(entries) > 1:
            for index, entry in enumerate(entries, 1):
                entry['title'] += f', part {index}'

        return entries

    def get_date(self):
        date_str = [item['data-date'] for item in self.soup.find_all() if 'data-date' in item.attrs][0]
        # refactor
        date = datetime.datetime.strptime(date_str, '%d.%m.%Y %H:%M')
        return date

    def get_title(self):
        # TODO: scrape title from web?
        match = re.match(self._VALID_URL, self.url)
        return match.group('title').replace('-', ' ')

    def get_show_name(self):
        # TODO: scrape show_name from web?
        match = re.match(self._VALID_URL, self.url)
        return match.group('show_name').replace('-', ' ')

    def extract(self):
        return self.get_entries()



from typing import Iterable, List, Type

from rtv.extractors import EXTRACTORS
from rtv.extractors.common import Extractor
from rtv.utils import validate_url
from rtv.video import Video


class RTVdownloader:
    extractors: List[Type[Extractor]]
    videos: List[Video]

    def __init__(self) -> None:
        self.extractors = EXTRACTORS
        self.videos = []

    def load(self, urls: Iterable[str]) -> None:
        for url in urls:
            if not validate_url(url):
                print(f'This is not a valid url: {url}, skipping...')
                continue

            for Extractor in self.extractors:
                if not Extractor.validate_url(url):
                    continue

                extractor = Extractor(url)
                extractor.run()
                self.videos.extend(extractor.videos)
                break
            else:
                print(f'None of the extractors can handle this url: {url}')

    def download(self, **kwargs) -> None:
        """
        Download each of the loaded videos.

        Args:
            **kwargs: Optional arguments that VideoDownloader takes:
                quality (str): Quality of the video ('best'/'worst')
                download_dir (str): Destination directory for the downloaded video.
                templates (dict): Dictionary of templates needed to generate a download path.

        """
        for video in self.videos:
            video.download(**kwargs)



import json
import re
from collections import namedtuple
from datetime import datetime

import requests
from bs4 import BeautifulSoup

from rtv.exceptions import VideoIdNotMatchedError
from rtv.extractors.common import (
    Entries, Extractor, GenericDescriptionMixin, GenericTitleMixin
)


class TokFm(GenericTitleMixin, GenericDescriptionMixin, Extractor):
    SITE_NAME = 'tokfm.pl'
    _VALID_URL = r'https?://(?:www\.)?audycje\.tokfm\.pl/.*/(?P<video_id>[a-z0-9-]*)'
    VideoInfo = namedtuple('VideoInfo', ('date', 'showname', 'host', 'guests'))

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')
        self.video_id = self._extract_id()
        self.info = self._scrape_info()

    def get_real_url(self) -> str:
        data = {'pid': self.video_id, 'st': 'tokfm'}
        r = requests.post('http://audycje.tokfm.pl/gets',
                          data=json.dumps(data),
                          cookies=self.response.cookies)
        url = json.loads(r.text)['url']
        return url

    def _extract_id(self) -> str:
        """
        Get video_id needed to obtain the real_url of the video.

        Raises:
            VideoIdNotMatchedError: If video_id is not matched with regular expression.

        """
        match = re.match(self._VALID_URL, self.url)

        if match:
            return match.group('video_id')
        else:
            raise VideoIdNotMatchedError

    @staticmethod
    def _process_info(raw_info: VideoInfo) -> VideoInfo:
        """Process raw information about the video (parse date, etc.)."""
        raw_date = raw_info.date
        date = datetime.strptime(raw_date, '%Y-%m-%d %H:%M')  # 2018-04-05 17:00
        video_info = raw_info._replace(date=date)
        return video_info

    def _scrape_info(self) -> VideoInfo:
        rows = self.soup.select('.tok-divTableRow')[:-2]  # omit duration SocialMedia links
        cells = [row.select('.tok-divTableCell')[1] for row in rows]
        cells[1].find('span').decompose()  # delete "Obserwuj" text

        # TODO: Guests have too much spaces between each guest
        # http://audycje.tokfm.pl/podcast/KRS-wybiera-nowych-sedziow-Sadu-Najwyzszego-Kaminski-Intencje-od-poczatku-byly-jasne-i-od-poczatku-byly-zle/66219
        data = [cell.get_text(strip=True) for cell in cells]
        raw_info = self.VideoInfo(*data)
        video_info = self._process_info(raw_info)
        return video_info

    def extract(self) -> Entries:
        entries = [{
            **self.info._asdict(),
            'title': self.get_title(),
            'description': self.get_description(),
            'url': self.get_real_url(),
            'ext': 'mp3'
        }]

        return entries



import datetime

from bs4 import BeautifulSoup

from rtv.extractors.common import Extractor


class PolskieRadio(Extractor):
    SITE_NAME = 'polskieradio.pl'
    _VALID_URL = r'https?://(?:www\.)?polskieradio\.pl/'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_html()
        self.soup = BeautifulSoup(self.html, 'lxml')

    def get_date(self):
        # polskieradio puts date into different tags
        date_formats = [
            (('span', ), {'class_': 'time'}, '%d.%m.%Y %H:%M'),
            (('span', ), {'class_': 'date'}, '%d/%m/%Y')
        ]
        for d in date_formats:
            try:
                date_str = self.soup.find(*d[0], **d[1]).text.strip()
                return datetime.datetime.strptime(date_str, d[2])
            except (ValueError, AttributeError):
                pass

    def extract(self):
        video_info = self.get_info()
        entries = video_info['entries']

        for entry in entries:
            entry.update({
                'date': self.get_date()
            })

        return entries

# TODO: Add support for all videos on this site.
# https://www.polskieradio.pl/10/5370/Artykul/1934652,Bedzie-glosno-24-listopada-godz-1803
# TODO: Scrape show name



import pprint

from rtv.downloaders.youtubedl import YoutubePD
from rtv.utils import clean_video_data


class Field:
    def __init__(self):
        self.name = None

    def __get__(self, instance, owner):
        if instance is None:
            return self
        return instance.data.get(self.name)

    def __set__(self, instance, value):
        raise AttributeError(f"You can't set {self.name} attribute manually.")


class Meta(type):
    def __new__(mcs, name, bases, class_dict):
        for key, value in class_dict.items():
            if isinstance(value, Field):
                value.name = key
        cls = type.__new__(mcs, name, bases, class_dict)
        return cls


class Video(metaclass=Meta):
    title = Field()
    date = Field()
    url = Field()
    ext = Field()

    def __init__(self, data):
        self.data = clean_video_data(data)

    def download(self, **kwargs):
        ypd = YoutubePD(self, **kwargs)
        ypd.download()

    def print_data(self):
        """Pretty print the data of this video."""
        pprint.pprint(self.data)

    def __str__(self):
        return f'Video({self.title})'

    def __repr__(self):
        return f'{self.__class__.__name__}({self.data})'



import contextlib
import os
import re
import sys
import urllib.parse

import tldextract
import validators

from rtv.exceptions import WrongUrlError


class DevNull:
    """
    DevNull class that has a no-op write and flush method.
    """
    def write(self, *args, **kwargs):
        pass

    def flush(self):
        pass


@contextlib.contextmanager
def suppress_stdout():
    """
    Context manager that suppresses stdout.

    Examples:
        >>> with suppress_stdout():
        ...     print('Test print')

        >>> print('test')
        test

    """
    save_stdout = sys.stdout
    sys.stdout = DevNull()
    yield
    sys.stdout = save_stdout


def validate_url(url):
    """
    Validate url using validators package.

    Args:
        url (str): Url.

    Returns:
        bool: True if valid, False otherwise.

    Examples:
        >>> validate_url('http://google.com')
        True

        >>> validate_url('http://google')  # doctest: +ELLIPSIS
        ValidationFailure(...)

        >>> if not validate_url('http://google'):
        ...     print('not valid')
        not valid

    """
    return validators.url(url)


def get_domain_name(url):
    """
    Extract a domain name from the url (without subdomain).

    Args:
        url (str): Url.

    Returns:
        str: Domain name.

    Raises:
        DomainNotMatchedError: If url is wrong.

    Examples:
        >>> get_domain_name('https://vod.tvp.pl/video/')
        'tvp.pl'

        >>> get_domain_name('https://vod')
        Traceback (most recent call last):
        ...
        rtv.exceptions.WrongUrlError: Couldn't match domain name of this url: https://vod

    """
    if not validate_url(url):
        raise WrongUrlError(f'Couldn\'t match domain name of this url: {url}')

    ext = tldextract.extract(url)
    return f'{ext.domain}.{ext.suffix}'


def clean_video_data(_data):
    """
    Clean video data:
        -> cleans title
        -> ...

    Args:
        _data (dict): Information about the video.

    Returns:
        dict: Refined video data.

    """

    data = _data.copy()

    # TODO: fix this ugliness
    title = data.get('title')
    if title:
        data['title'] = clean_title(title)

    return data


def clean_title(title):
    """
    Clean title -> remove dates, remove duplicated spaces and strip title.

    Args:
        title (str): Title.

    Returns:
        str: Clean title without dates, duplicated, trailing and leading spaces.

    """
    date_pattern = re.compile(r'\W*'
                              r'\d{1,2}'
                              r'[/\-.]'
                              r'\d{1,2}'
                              r'[/\-.]'
                              r'(?=\d*)(?:.{4}|.{2})'
                              r'\W*')
    title = date_pattern.sub(' ', title)
    title = re.sub(r'\s{2,}', ' ', title)
    title = title.strip()
    return title


def clean_filename(filename):
    """
    Remove unsupported filename characters.
    On Windows file names cannot contain any of \/:*?"<>| characters.
    Effectively remove all characters except alphanumeric, -_#.,() and spaces.

    Args:
        filename (str): Name of a file.

    Returns:
        str: Filename without unsupported characters.

    """
    return re.sub('[^\w\-_#.,() ]', '', filename)


def file_exists(path):
    """
    Check whether a file exists.

    Args:
        path (str): Path to a file.

    Returns:
        bool: True if exists, False otherwise.

    """
    return os.path.exists(path)


def get_ext(url):
    """
    Extract an extension from the url.

    Args:
        url (str): String representation of a url.

    Returns:
        str: Filename extension from a url (without a dot), '' if extension is not present.

    """

    parsed = urllib.parse.urlparse(url)
    root, ext = os.path.splitext(parsed.path)
    return ext.lstrip('.')


def delete_duplicates(seq):
    """
    Remove duplicates from an iterable, preserving the order.

    Args:
        seq: Iterable of various type.

    Returns:
        list: List of unique objects.

    """
    seen = set()
    seen_add = seen.add
    return [x for x in seq if not (x in seen or seen_add(x))]



import argparse
from pathlib import Path

DEFAULT_OPTIONS = {
    'download_dir': Path.home() / 'Desktop' / 'RTV',
    'templates': {
        'ipla.tv': '{date:%d} {title}.{ext}',
        'polsatnews.pl': '{date:%d} {title}.{ext}',
        'polskieradio.pl': '{date:%d} {title}.{ext}',
        'radiozet.pl': '{date:%d} {show_name} - {title}.{ext}',
        'rmf24.pl': '{date:%d} {title}.{ext}',
        'tokfm.pl': '{date:%d} {title}.{ext}',
        'tvn24.pl': '{date:%d} {title}.{ext}',
        'tvp.info': '{date:%d} {title}.{ext}',
        'tvp.pl': '{date:%d} {title}.{ext}',
        'tvpparlament.pl': '{date:%d} {title}.{ext}',
        'vod.pl': '{date:%d} {show_name} - {title}.{ext}',
        'wp.pl': '{date:%d} {show_name} - {title}.{ext}'
    },
    'quality': 'worst'
}


def parse_options():
    """
    Parse command line arguments.

    Returns:
        options, args

    """
    parser = argparse.ArgumentParser(description='Video downloader by radzak.',
                                     prog='RTVdownloader')
    urls_group = parser.add_mutually_exclusive_group(required=True)
    urls_group.add_argument('urls',
                            type=str,
                            metavar='URL',
                            default=[],
                            nargs='*',
                            help='urls of sites containing videos you wish to download'
                            )

    urls_group.add_argument('-f',
                            type=argparse.FileType('r'),
                            dest='files',
                            metavar='FILE',
                            default=[],
                            nargs='*',
                            help='text file with urls of sites containing videos you '
                                 'wish to download '
                            )

    urls_group.add_argument('-o',
                            type=str,
                            dest='onetabs',
                            metavar='ONETAB',
                            default=[],
                            nargs='*',
                            help='onetab links containing urls of the videos you wish to download'
                            )

    options = DEFAULT_OPTIONS

    # TODO: add dir option that defaults to the DEFAULT_OPTIONS['dl_path']

    args = parser.parse_args()
    return options, args



from rtv.onetab import get_urls_from_onetab
from rtv.options import parse_options
from rtv.rtvdownloader import RTVdownloader


def main():
    options, args = parse_options()
    urls = args.urls
    files = args.files
    onetabs = args.onetabs

    rtv = RTVdownloader()

    for file in files:
        file_urls = filter(None, (line.strip() for line in file))
        urls.extend(file_urls)

    for onetab in onetabs:
        onetab_urls = get_urls_from_onetab(onetab)
        urls.extend(onetab_urls)

    rtv.load(set(urls))
    rtv.download(**options)


if __name__ == '__main__':
    main()

# TODO: add generic downloader?
# TODO: add metadata to file, such as description ect., check if it is already there
# TODO: add max download time and retry?
# TODO: https://www.tvn24.pl/loza-prasowa,25,m/loza-prasowa-17-12-2017,799547.html



import shlex
import threading

import youtube_dl

from rtv.downloaders.common import VideoDownloader


class YoutubePD(VideoDownloader):
    def _real_download(self, path):
        # TODO: choose the right url and extension if the 'formats' key is present in video data
        quality = self.quality
        url = self.video.url
        ext = self.video.ext

        def run():
            command = (
                f'youtube-dl '
                f'-f {quality}[ext={ext}]/'
                f'{quality}video+bestaudio/bestaudio '
                f'--merge-output-format "{ext}" '
                f'-o "{path}" '
                f'{url}'
            )
            youtube_dl.main(shlex.split(command)[1:])

        t = threading.Thread(target=run)
        t.start()
        t.join()



__title__ = 'rtv-downloader'
__description__ = 'Command-line program to download videos from various sites.'
__url__ = 'https://github.com/radzak/rtv-downloader'
__version__ = '2.0.7'
__author__ = 'Radek Krzak'
__author_email__ = 'radek.krzak@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018 Radek Krzak'



from pathlib import Path

from setuptools import setup

packages = ['rtv', 'rtv.extractors', 'rtv.downloaders']

requires = [
    'beautifulsoup4',
    'dateparser',
    'Js2Py',
    'lxml',
    'requests',
    'tldextract'
    'validators',
    'youtube_dl',
]

about = {}
with Path('rtv', '__version__.py').open('r', encoding='utf-8') as f:
    exec(f.read(), about)

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__description__'],
    url=about['__url__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    license=about['__license__'],
    packages=packages,
    entry_points={
        'console_scripts': ['rtv=rtv.cli:main'],
    },
    python_requires='>=3.6',
    install_requires=requires,
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: End Users/Desktop',
        'Topic :: Multimedia :: Video',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ]
)



import os

from rtv.exceptions import NoTemplateFoundError, WrongQualityError
from rtv.options import DEFAULT_OPTIONS
from rtv.utils import clean_filename


class VideoDownloader:
    def __init__(self, video, quality=None, download_dir=None, templates=None) -> None:
        """
        Create a VideoDownloader for a given video.

        Args:
            video (Video): Video object.
            quality (str): Quality of the video (best/worst). Audio quality defaults to best.
            download_dir (str): Destination directory for the downloaded video.
            templates (dict): Dictionary of templates needed to generate a download path.

        """
        self.video = video
        self.quality = quality or DEFAULT_OPTIONS['quality']
        self.download_dir = download_dir or DEFAULT_OPTIONS['download_dir']
        self.templates = templates or DEFAULT_OPTIONS['templates']

        if self.quality not in ('worst', 'best'):
            raise WrongQualityError

    def _real_download(self, path) -> None:
        """Real download process. Redefine in subclasses."""
        raise NotImplementedError('This method must be implemented by subclasses')

    def download(self) -> None:
        """Download video to target location. Choose worst quality by default, to decrease file size."""
        path = self.render_path()
        self._real_download(path)

    def render_path(self) -> str:
        """Render path by filling the path template with video information."""
        # TODO: Fix defaults when date is not found (empty string or None)
        # https://stackoverflow.com/questions/23407295/default-kwarg-values-for-pythons-str-format-method

        from string import Formatter

        class UnseenFormatter(Formatter):
            def get_value(self, key, args, kwds):
                if isinstance(key, str):
                    try:
                        return kwds[key]
                    except KeyError:
                        return key
                else:
                    return super().get_value(key, args, kwds)

        data = self.video.data
        site_name = data['site']

        try:
            template = self.templates[site_name]
        except KeyError:
            raise NoTemplateFoundError

        fmt = UnseenFormatter()
        filename_raw = fmt.format(template, **data)
        filename = clean_filename(filename_raw)
        path = os.path.join(self.download_dir, filename)
        return path



class RTVException(Exception):
    pass


class VideoIdNotMatchedError(RTVException):
    pass


class WrongUrlError(RTVException):
    pass


class WrongQualityError(RTVException):
    pass


class NoTemplateFoundError(RTVException):
    pass
